// Copyright 2025 Georges Racinet <georges.racinet@cloudcrane.io>
//
// This software may be used and distributed according to the terms of the
// GNU General Public License version 2 or any later version.
// SPDX-License-Identifier: GPL-2.0-or-later
use std::sync::Arc;

use tokio_util::sync::CancellationToken;
use tonic::{Request, Response};

use crate::config::Config;
use crate::hgitaly::mercurial_changeset_service_client::MercurialChangesetServiceClient;
use crate::hgitaly::mercurial_changeset_service_server::{
    MercurialChangesetService, MercurialChangesetServiceServer,
};
use crate::hgitaly::{ListMercurialChangesetsRequest, ListMercurialChangesetsResponse};
use crate::sidecar;
use crate::streaming::ResultResponseStream;

#[derive(Debug)]
#[allow(dead_code)]
pub struct MercurialChangesetServiceImpl {
    config: Arc<Config>,
    shutdown_token: CancellationToken,
    sidecar_servers: Arc<sidecar::Servers>,
}

#[tonic::async_trait]
impl MercurialChangesetService for MercurialChangesetServiceImpl {
    async fn list_mercurial_changesets(
        &self,
        request: Request<ListMercurialChangesetsRequest>,
    ) -> ResultResponseStream<ListMercurialChangesetsResponse> {
        sidecar::server_streaming!(
            self,
            request,
            MercurialChangesetServiceClient,
            list_mercurial_changesets
        )
    }
}

/// Takes care of boilerplate that would instead be in the startup sequence.
pub fn mercurial_changeset_server(
    config: &Arc<Config>,
    shutdown_token: &CancellationToken,
    sidecar_servers: &Arc<sidecar::Servers>,
) -> MercurialChangesetServiceServer<MercurialChangesetServiceImpl> {
    MercurialChangesetServiceServer::new(MercurialChangesetServiceImpl {
        config: config.clone(),
        shutdown_token: shutdown_token.clone(),
        sidecar_servers: sidecar_servers.clone(),
    })
}
